/*-----------------------------------------------------------------------------
# Nome:        ServidorOpenChat.java
# Finalidade: Servidor
#
# Author:	Cesar Kallas			cesarkallas at gmx.net 			http://cesarkallas.soulivre.org
#			Renan Fernanades 	renanfernandes at gmail.com	http://renanfernandes.soulivre.org
# Copyright:   (c) 2005
# Licence:     GNU/GPL
# Pontificia Univerisidade Catolica - Puc Campinas - Brasil
#-----------------------------------------------------------------------------*/

package openchat.servidor;
import java.net.Socket;
import java.net.ServerSocket;
import java.io.*;
import java.util.*;
import java.lang.*;
import openchat.bd.BancoDados;
import openchat.servidor.Mensagem;

/*	
 *	Classe que recebe a porta do servidor a ser aberto e os dados para conectar no banco de dados
 */
public class ServidorOpenChat
{
	private ServerSocket sv_sckt_canal;
	private Socket sckt_cliente;
	
	public ServidorOpenChat(int servidorPorta, String bancoDadosIP,  int bancoDadosPorta, String bancoDadosUsuario, 
		String bancoDadosSenha, String bancoDadosEsquema)
	{
		BancoDados BD = new BancoDados(bancoDadosIP, bancoDadosPorta, bancoDadosUsuario, bancoDadosSenha, bancoDadosEsquema);
		BD.AtualizaIpPorta();
//		BD.ApagaTudo();
		try {
			sv_sckt_canal = new ServerSocket(servidorPorta);
			for(;;) {
				System.out.println("\n\t[ Servidor OpenChat :: Porta " + servidorPorta + " ]");
				sckt_cliente = sv_sckt_canal.accept();
				Thread thread = new Thread(new ServidorTrataRequisicoes(sckt_cliente,
					bancoDadosIP, bancoDadosPorta, bancoDadosUsuario, bancoDadosSenha, bancoDadosEsquema));
				thread.start();
			}
		}
		catch(IOException sckt_servidor) {
			System.err.println("Falha: Canal de Comunicacao @ ServidorOpenChat" + sckt_servidor.getMessage());
		}
	}
}

/*
 *	Classe Servidor
 * 	Essa classe que recebe as mensagens enviadas pelo cliente e as trata
 */
class ServidorTrataRequisicoes implements Runnable 
{
	private Socket sckt_cliente;
	private String Host, TOKEN="~~~~~~~~~~~~~~~";
	private StringTokenizer msg_cliente;
	private BufferedReader  sckt_cliente_entrada;
	private PrintWriter sckt_cliente_saida;
	private BancoDados BD;
	private Mensagem SM;

	// Cria conexao com banco de dados e com a classe que envia mensagens
	public ServidorTrataRequisicoes(Socket sckt_cliente, String bancoDadosIP, int bancoDadosPorta, String bancoDadosUsuario, String bancoDadosSenha, String bancoDadosEsquema)
	{
		this.sckt_cliente = sckt_cliente;
		BD = new BancoDados(bancoDadosIP, bancoDadosPorta, bancoDadosUsuario, bancoDadosSenha, bancoDadosEsquema);
		SM = new Mensagem(bancoDadosIP, bancoDadosPorta, bancoDadosUsuario, bancoDadosSenha, bancoDadosEsquema);
	}
	
	// Funcao que trata as requisicoes ~ e' uma thread
	public void run()
	{
		String requisicao_cliente, login, email, nome, senha, cidade, informacao, ip, frase_secreta;
		int id, porta, contato;
		
		try {
			sckt_cliente_entrada = new BufferedReader (new InputStreamReader (this.sckt_cliente.getInputStream()));
			msg_cliente = new StringTokenizer (sckt_cliente_entrada.readLine(), TOKEN);
			sckt_cliente_saida = new PrintWriter (new OutputStreamWriter(this.sckt_cliente.getOutputStream()));
		}
		catch(IOException ErroRun) {
			System.out.println("Erro ao instanciar socket de reposta ao cliente" + ErroRun.getMessage());
		}	
	
		try {
			requisicao_cliente = msg_cliente.nextToken();
			System.out.println("Requisicao :: " + requisicao_cliente);

			if(requisicao_cliente.equals("NewUser")) {
				login = msg_cliente.nextToken();
				email = msg_cliente.nextToken();
				nome = msg_cliente.nextToken();
				senha = msg_cliente.nextToken();
				cidade = msg_cliente.nextToken();
				frase_secreta = msg_cliente.nextToken();
				informacao = msg_cliente.nextToken();

				sckt_cliente_saida.print(BD.NewUser(login, email, nome, senha, cidade, frase_secreta, informacao));
				sckt_cliente_saida.flush();
			}
	
			if(requisicao_cliente.equals("QtUsers")) {
				sckt_cliente_saida.print(BD.Qtusuarios());
				sckt_cliente_saida.flush();
			}		
		
      if(requisicao_cliente.equals("UpdateUserPort")) {
        id = Integer.parseInt(msg_cliente.nextToken());
        porta = Integer.parseInt(msg_cliente.nextToken());
        BD.UpdateUserPort(id, porta);
      }
      
			if(requisicao_cliente.equals("Login")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				senha = msg_cliente.nextToken();
				porta = Integer.parseInt(msg_cliente.nextToken());
				ip = sckt_cliente.getInetAddress().getHostAddress();
				String LoginTeste = "" + BD.UserLogin(id, senha, ip, porta);
				
				sckt_cliente_saida.println(LoginTeste);
				sckt_cliente_saida.flush();
				sckt_cliente_saida.println(BD.ReturnUserLogin(id));
				sckt_cliente_saida.flush();

				if(LoginTeste.equals("true")) {
					StringTokenizer UserOn = new StringTokenizer(BD.ReturnContactList(id), TOKEN);
					UserOn.nextToken();
					int IdContact;
					String IpContact;
					while(UserOn.hasMoreTokens()) {
						IdContact = Integer.parseInt(UserOn.nextToken());
						UserOn.nextToken();
						IpContact = UserOn.nextToken();
						UserOn.nextToken();
						if(!IpContact.equals("0"))
							new ContatoAtualizaLista(IpContact,BD.ReturnUserPort(IdContact)).start();
					}
				}
			}
		
			if(requisicao_cliente.equals("Logout")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				senha = msg_cliente.nextToken();
				String Test = "" + BD.UserLogout(id, senha);
				sckt_cliente_saida.print(Test);
				sckt_cliente_saida.flush();

				if(Test.equals("true")) {
					StringTokenizer UserOff = new StringTokenizer(BD.ReturnContactList(id), TOKEN);
					UserOff.nextToken();
					int IdContact;
					String IpContact;
	
					while(UserOff.hasMoreTokens())
					{
						IdContact = Integer.parseInt(UserOff.nextToken());
						UserOff.nextToken();
						IpContact = UserOff.nextToken();
						UserOff.nextToken();
						if(!IpContact.equals("0"))
							new ContatoAtualizaLista(IpContact,BD.ReturnUserPort(IdContact)).start();
					}
				}
			}
		
			if(requisicao_cliente.equals("DeleteUser")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				senha = msg_cliente.nextToken();
				sckt_cliente_saida.print(BD.DeleteUser(id, senha));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("UpdateUser")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				senha = msg_cliente.nextToken();
				login = msg_cliente.nextToken();
				email = msg_cliente.nextToken();
				nome = msg_cliente.nextToken();
				cidade = msg_cliente.nextToken();
				informacao = msg_cliente.nextToken();
				frase_secreta = msg_cliente.nextToken();
				
				sckt_cliente_saida.print(BD.UpdateUser(id, login, email, nome, senha, cidade, informacao, frase_secreta));
				sckt_cliente_saida.flush();
			}		

			if(requisicao_cliente.equals("ConfFraseScreta")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				frase_secreta = msg_cliente.nextToken();
				sckt_cliente_saida.println(BD.ConfFraseScreta(id, frase_secreta));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserLogin")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserLogin(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserIp")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserIp(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserName")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserName(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserEmail")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserEmail(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserInfo")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserInfo(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserCity")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserCity(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserId")) {
				email = msg_cliente.nextToken();
				sckt_cliente_saida.print(BD.ReturnUserId(email));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserPort")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUserPort(id));
				sckt_cliente_saida.flush();
			}
					
			if(requisicao_cliente.equals("TestUserIdent"))	{
				id = Integer.parseInt(msg_cliente.nextToken());
				senha = msg_cliente.nextToken();
				
				sckt_cliente_saida.print(BD.TestUserIdent(id, senha));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUserContactInformation")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				
				String Information = BD.ReturnUserLogin(id)+TOKEN+
						BD.ReturnUserEmail(id)+TOKEN+
						BD.ReturnUserCity(id)+TOKEN+
						BD.ReturnUserName(id)+TOKEN+
						BD.ReturnUserInfo(id)+TOKEN+
						BD.ReturnFraseSecreta(id)+TOKEN;
				sckt_cliente_saida.print(Information);
				sckt_cliente_saida.flush();
			}
				
			if(requisicao_cliente.equals("ExistUser")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ExistUser(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("AddContact")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				contato = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.AddUserContact(id, contato));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("DeleteContact")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				contato = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.DeleteUserContact(id, contato));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("SendMessage")) {
				int IdFrom = Integer.parseInt(msg_cliente.nextToken());
				int IdTo = Integer.parseInt(msg_cliente.nextToken());
				
				String Msg = "";
				
				try {
					for (;;) {
						Msg = Msg + sckt_cliente_entrada.readLine();
						if(sckt_cliente_entrada.ready())
							Msg = Msg + '\n';
						else
							break; // caso nao existam, sair do loop de montagem da mensagem
					}
				}
				catch(Exception E) {
					System.out.println("ERRO NO FOR SENDMESSAGE :"+E.getMessage());
				}

				sckt_cliente_saida.print(SM.SendTo(IdFrom, IdTo, Msg));
				sckt_cliente_saida.flush();
			}
				
			if(requisicao_cliente.equals("ExistContactUser")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				contato = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ExistContactUser(id, contato));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnContactList")) {
				id = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.println(BD.ReturnContactList(id));
				sckt_cliente_saida.flush();
			}
			
			if(requisicao_cliente.equals("ReturnUsers")) {
				int option = Integer.parseInt(msg_cliente.nextToken());
				sckt_cliente_saida.print(BD.ReturnUsers(option));
				sckt_cliente_saida.flush();
			}
		}
		catch(Exception NotFound) {
			System.err.println("Requisicao invalida" + NotFound.getMessage());
		}

		try {
			sckt_cliente_entrada.close();
			sckt_cliente_saida.close();
			sckt_cliente.close();
		}
		catch(IOException CloseSockets) {
			System.out.println("Falha ao fechar socket do servidor" + CloseSockets.getMessage());
		}
	}
}

/*
 *	Classe que atualiza a lista de contatos do usuario
 */
class ContatoAtualizaLista extends Thread
{
	private String Up="List", Token="~~~~~~~~~~~~~~~", IpTo;
	private int PortTo;
	private Socket sckt_MSG;
	private PrintWriter Send;
	
	public ContatoAtualizaLista(String IpTo, int PortTo)
	{
		this.IpTo = IpTo;
		this.PortTo = PortTo;
	}
	
	public void run()
	{
		try {
			sckt_MSG = new Socket(IpTo, PortTo);
			Send = new PrintWriter(sckt_MSG	.getOutputStream());
			Send.print(Up+Token);
			Send.flush();

			Send.close();
			sckt_MSG.close();
			sckt_MSG = null;
			Send = null;
		}
		catch(IOException UP) {
			System.out.println("errro atualizando lista: "+ UP.getMessage());
		}
	}
}
